# -*- coding: utf-8 -*-
"""
Created on Fri Oct 28 17:14:51 2016

@author: vgor
"""
from pandas import read_csv
from subprocess import call
from os import path
from statsmodels.nonparametric.smoothers_lowess import lowess
from scipy.stats import binned_statistic
import pylab as plt
import numpy as np
import sys

##Path to ScanHeadsman
##Change it to fit
shPath = ".\\ScanHeadsman.exe"


def process(filename):
    df = read_csv(filename)
    
    #convert MS data into numpy arrays
    df["mzArray"] = df["mzArray"].apply(lambda t: np.fromstring(t, sep = " "))
    df["intArray"] = df["intArray"].apply(lambda t: np.log10(np.fromstring(t, sep = " ")))
    
    #min and max intensity
    df["minInt"] = df["intArray"].apply(np.min)
    df["maxInt"] = df["intArray"].apply(np.max)
    df["meanInt"] = df["intArray"].apply(np.mean)
    
    #split MS1 and MS2 scans
    df1 = df[df["MSOrder"] == 1]    
    df2 = df[df["MSOrder"] > 1]
    
    #calculate relative intensities, collect counts from the histogram with 100 bins
    #and calculate average values for all histograms
    plt.figure("Normalized intensity histogram", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Normalized Ion Intensity Distribution")
    avHist = np.mean((df1["intArray"] / df1["maxInt"]).apply(lambda a: np.histogram(a, 100)[0]), axis = 0)
    plt.plot(np.linspace(0, 100, 100), avHist/np.max(avHist), color = 'c', lw = 3, label = "MS1 scans")
    avHist = np.mean((df2["intArray"] / df2["maxInt"]).apply(lambda a: np.histogram(a, 100)[0]), axis = 0)
    plt.plot(np.linspace(0, 100, 100), avHist/np.max(avHist), color = 'g', lw = 3, label = "MSn scans")
    plt.xlabel("Percent bin of log10(Intensity)")
    plt.ylabel("Normalized number of ions in the bin")
    plt.legend()
    
    plt.figure("Spectrum Intensity", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Spectrum Intensity")
    plt.scatter(df1["meanInt"], df1["maxInt"], s = 5, alpha = 0.3, color = "c", label = "MS1 scans")
    plt.scatter(df1["meanInt"], df1["minInt"], s = 5, alpha = 0.3, color = "c")
    plt.scatter(df2["meanInt"], df2["maxInt"], s = 5, alpha = 0.3, color = "g", label = "MSn scans")
    plt.scatter(df2["meanInt"], df2["minInt"], s = 5, alpha = 0.3, color = "g")
    avgMin = binned_statistic(df["meanInt"], df["minInt"], bins = 100)[0]
    avgMax, binEdges, binNumber = binned_statistic(df["meanInt"], df["maxInt"], bins = 100)
    avgMean = (binEdges[1:] + binEdges[:-1])/2
    plt.plot(avgMean, avgMax, "k-", lw = 3)
    plt.plot(avgMean, avgMin, "k-", lw = 3)
    plt.xlabel("log10(Intensity)")
    plt.ylabel("log10(Intensity)")
    plt.legend(loc = "upper left")

    plt.figure("Dynamic Range", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Dynamic Range")
    plt.scatter(df1["meanInt"], df1["maxInt"] - df1["minInt"], s = 5, alpha = 0.3, color = "c", label = "MS1 scans")
    plt.scatter(df2["meanInt"], df2["maxInt"] - df2["minInt"], s = 5, alpha = 0.3, color = "g", label = "MSn scans")
    plt.plot(avgMean, avgMax - avgMin, "k-", lw = 3, label = "Dynamic range")
    plt.xlabel("log10(Intensity)")
    plt.ylabel("log10(Max/Min)")
    plt.ylim(ymin = 0)
    plt.legend(loc = "upper left")
    
    avTopN = 1.0 * sum(df["MSOrder"] != 1)/sum(df["MSOrder"] == 1)
    
    plt.figure("Cycle Time vs RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Cycle Time vs RT [elution time]")
    ms1times = df.loc[df["MSOrder"] == 1, u"RT [min]"].values
    plt.scatter(ms1times[:-1], 60. * (ms1times[1:] - ms1times[:-1]), s = 5, alpha = 0.7)
    lowessFit = lowess(60. *(ms1times[1:] - ms1times[:-1]), ms1times[:-1], is_sorted=True, frac=0.025, it=0)
    plt.plot(lowessFit[:, 0], lowessFit[:, 1], "r-", lw = 3, label = "Average Cycle Time")
    plt.xlabel("RT (min)")
    plt.ylabel("Cycle Time (s)")
    plt.legend()
    plt.ylim(ymin = 0)
    plt.xlim((ms1times[0], ms1times[-1]))

    plt.figure("TopN vs RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("TopN vs RT [elution time]")
    ms1scans = df.loc[df["MSOrder"] == 1, u"Scan"].values
    plt.scatter(ms1times[:-1], (ms1scans[1:] - ms1scans[:-1]) - 1, s = 5, alpha = 0.7)
    lowessFit = lowess((ms1scans[1:] - ms1scans[:-1]) - 1, ms1times[:-1], is_sorted=True, frac=0.025, it=0)
    plt.plot(lowessFit[:, 0], lowessFit[:, 1], "r-", lw = 3, label = "Average TopN")
    plt.axhline(avTopN, color = "c", lw = 3, label = "Mean TopN")
    plt.text(ms1times[-1]*0.99, avTopN*1.04, "{:.3f}".format(avTopN), ha = "right", color = "c", fontsize = 14)
    plt.xlabel("RT (min)")
    plt.ylabel("TopN")
    plt.legend()
    plt.ylim(ymin = -1)
    plt.xlim((ms1times[0], ms1times[-1]))
    
    plt.figure("IIT-MS1 at RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("IIT-MS1 vs RT [all time]")
    plt.scatter(df.loc[df["MSOrder"] == 1, "RT [min]"].values,
            df.loc[df["MSOrder"] == 1, "Ion Injection Time (ms)"].values, alpha = 0.7)
    lowessFit = lowess(df.loc[df["MSOrder"] == 1, "Ion Injection Time (ms)"].values,
                        df.loc[df["MSOrder"] == 1, "RT [min]"].values, is_sorted=True, frac=0.02, it=0)
    plt.plot(lowessFit[:, 0], lowessFit[:, 1], "r-", lw = 3, label = "Average IIT")
    plt.xlabel("Retention Time (min)")
    plt.ylabel("MS1 ion injection time (ms)")
    plt.ylim(ymin = -1)
    plt.xlim(xmin = 0, xmax = max(df.loc[:, "RT [min]"].values))
    plt.legend()
    
    plt.figure("IIT-MS2 at RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("IIT-MS2 vs RT [all time]")
    plt.scatter(df.loc[df["MSOrder"] == 2, "RT [min]"].values,
            df.loc[df["MSOrder"] == 2, "Ion Injection Time (ms)"].values, alpha = 0.7)
    lowessFit = lowess(df.loc[df["MSOrder"] == 2, "Ion Injection Time (ms)"].values,
                        df.loc[df["MSOrder"] == 2, "RT [min]"].values, is_sorted=True, frac=0.02, it=0)
    plt.plot(lowessFit[:, 0], lowessFit[:, 1], "r-", lw = 3, label = "Average IIT")
    plt.xlabel("Retention Time (min)")
    plt.ylabel("MS2 ion injection time (ms)")
    plt.ylim(ymin = -1)
    plt.xlim(xmin = 0, xmax = max(df.loc[:, "RT [min]"].values))
    plt.legend()
    
    plt.show()

def parseCMD(argv):
    """
    Parses vector of command line args
    """
    result = {}
    try:
        args = argv[1:]
        while len(args) > 0:
            k = args.pop(0)
            if k.lower() == "-force":
                result["Force"] = True
            elif k.lower() == "-rtmin":
                result["RTmin"] = float(args.pop(0))
            elif k.lower() == "-rtmax":
                result["RTmax"] = float(args.pop(0))
            elif not k.startswith("-"):
                result["Input"] = k
                        
    except:
        result = {}
    
    return result

if __name__ == "__main__":
    args = parseCMD(sys.argv)
    if len(args) > 0:
        if args["Input"].lower().endswith(".raw"):
            txtPath = path.splitext(args["Input"])[0] + ".txt"
            if (not path.exists(txtPath)) or "Force" in args.keys():
                print("It seems like ScanHeadsman was not executed yet\nHold on, I'll do it for you")
                call([shPath, args["Input"]])
            else:
                print("""It seems like ScanHeadsman was executed already\nI will use the old export file.\nUse -force as argument to regenerate the export file""")
                
        elif args["Input"].lower().endswith(".txt"):
            txtPath = args["Input"]
        else:
            print("No input specified\nI'm lost")
        
        try:
            process(txtPath)
        except Exception as e:
            print("Error: {}".format(repr(e)))
        
    else:
        print("Error parsing command line")
