# -*- coding: utf-8 -*-
"""
Created on Fri Oct 28 17:14:51 2016

@author: vgor
"""
from pandas import read_csv
from subprocess import call
from os import path
from statsmodels.nonparametric.smoothers_lowess import lowess
import pylab as plt
import numpy as np
import sys

##Path to ScanHeadsman
##Change it to fit
shPath = ".\\ScanHeadsman.exe"


def process(filename, rtMin, rtMax):
    df = read_csv(filename)
    
    print ("All time:")
    print ("#MS1: {}; #MS2: {}; TopN = {}".format(sum(df["MSOrder"] == 1), 
        sum(df["MSOrder"] != 1), 1.0 * sum(df["MSOrder"] != 1)/sum(df["MSOrder"] == 1)))
    
    vdf = df[np.logical_and(df["RT [min]"] > rtMin, df["RT [min]"] < rtMax)]

    print ("Elution time ({} -- {}):".format(rtMin, rtMax))
    print ("#MS1: {}; #MS2: {}; TopN = {}".format(sum(vdf["MSOrder"] == 1), 
        sum(vdf["MSOrder"] != 1), 1.0 * sum(vdf["MSOrder"] != 1)/sum(vdf["MSOrder"] == 1)))

    plt.figure("Cycle time", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Cycle length (s) [elution time]")
    ms1times = vdf.loc[vdf["MSOrder"] == 1, "RT [min]"].values
    plt.hist(60 * (ms1times[1:] - ms1times[:-1]), bins = 50)
    plt.xlabel("Cycle length (s)")
    plt.xlim(xmin = 0)
    
    plt.figure("Cycle Time vs RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Cycle Time vs RT [elution time]")
    ms1times = vdf.loc[vdf["MSOrder"] == 1, u"RT [min]"].values
    plt.scatter(ms1times[:-1], 60. * (ms1times[1:] - ms1times[:-1]), alpha = 0.7)
    lowessFit = lowess(60. *(ms1times[1:] - ms1times[:-1]), ms1times[:-1], is_sorted=True, frac=0.05, it=0)
    plt.plot(lowessFit[:, 0], lowessFit[:, 1], "r-", lw = 3, label = "Average Cycle Time")
    plt.xlabel("RT (min)")
    plt.ylabel("Cycle Time (s)")
    plt.legend()
    plt.ylim(ymin = 0)


    plt.figure("TopN vs RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("TopN vs RT [elution time]")
    ms1scans = vdf.loc[vdf["MSOrder"] == 1, u"Scan"].values
    plt.scatter(ms1times[:-1], (ms1scans[1:] - ms1scans[:-1]) - 1, alpha = 0.7)
    lowessFit = lowess((ms1scans[1:] - ms1scans[:-1]) - 1, ms1times[:-1], is_sorted=True, frac=0.05, it=0)
    plt.plot(lowessFit[:, 0], lowessFit[:, 1], "r-", lw = 3, label = "Average TopN")
    plt.xlabel("RT (min)")
    plt.ylabel("TopN")
    plt.legend()
    plt.ylim(ymin = -1)

    plt.figure("Ion injection time MS1", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Ion injection time MS1 [elution time]")
    plt.hist(vdf.loc[vdf["MSOrder"] == 1, "Ion Injection Time (ms)"].values, bins = 50)
    plt.xlabel("IIT (ms)")

    plt.figure("Ion injection time MS2", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Ion injection time MS2 [elution time]")
    plt.hist(vdf.loc[vdf["MSOrder"] == 2, "Ion Injection Time (ms)"].values, bins = 50)
    plt.xlabel("IIT (ms)")

    plt.figure("Precursor m/z", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Precursor m/z [elution time]")
    plt.hist(vdf.loc[vdf["MSOrder"] == 2, "Monoisotopic M/Z"].values, bins = 50)
    plt.xlabel("Precursor m/z (Th)")

    plt.figure("Charge", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("Charge [all time]")
    z = df.loc[df["MSOrder"] == 2, "Charge State"].values
    plt.hist(z, bins = range(int(min(z)), int(max(z)) + 2), align = "left", rwidth = 0.8)
    plt.xlabel("Charge state")

    plt.figure("MS1 peaks", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("MS1 peaks [elution time]")
    plt.hist(vdf.loc[vdf["MSOrder"] == 1, "NrCentroids"].values, bins = 50)
    plt.xlabel("# peaks")

    plt.figure("MS2 peaks", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("MS2 peaks [elution time]")
    plt.hist(vdf.loc[vdf["MSOrder"] == 2, "NrCentroids"].values, bins = 50)
    plt.xlabel("# peaks")

    plt.figure("IIT at RT", edgecolor = "w", facecolor = "w", figsize = (12, 9))
    plt.title("MS1-IIT vs RT [all time]")
    plt.scatter(df.loc[df["MSOrder"] == 1, "RT [min]"].values,
            df.loc[df["MSOrder"] == 1, "Ion Injection Time (ms)"].values, alpha = 0.7)
    plt.xlabel("Retention Time (min)")
    plt.ylabel("MS1 ion injection time (ms)")
    plt.ylim(ymin = -1)
    plt.xlim(xmin = 0, xmax = max(df.loc[df["MSOrder"] == 1, "RT [min]"].values))

    plt.show()

def parseCMD(argv):
    """
    Parses vector of command line args
    """
    result = {}
    try:
        args = argv[1:]
        while len(args) > 0:
            k = args.pop(0)
            if k.lower() == "-force":
                result["Force"] = True
            elif k.lower() == "-rtmin":
                result["RTmin"] = float(args.pop(0))
            elif k.lower() == "-rtmax":
                result["RTmax"] = float(args.pop(0))
            elif not k.startswith("-"):
                result["Input"] = k
                        
    except:
        result = {}
    
    return result


##Lumos QC 161111
#process("E:\\RawData\\LumosQC\\161111\\20161111_HELA_2colSetup_OTIT2.txt", 10, 100)
##Lumos QC 161005
#process("E:\\RawData\\LumosQC\\161005\\20161005_HELA_50cm_140min_55C_OTIT1.txt", 8, 100)
##Lumos QC 160906
#process("E:\\RawData\\LumosQC\\160906\\20160906_HELA_50cm_140min_55C_OTOT.txt", 8, 100)


if __name__ == "__main__":
    args = parseCMD(sys.argv)
    if len(args) > 0:
        if args["Input"].lower().endswith(".raw"):
            txtPath = path.splitext(args["Input"])[0] + ".txt"
            if (not path.exists(txtPath)) or "Force" in args.keys():
                print ("It seems like ScanHeadsman was not executed yet\nHold on, I'll do it for you")
                call([shPath, "--noMS", args["Input"]])
            else:
                print ("""It seems like ScanHeadsman was executed already
                I will use the old export file.
                Use -force as argument to regenerate the export file""")
                
        elif args["Input"].lower().endswith(".txt"):
            txtPath = args["Input"]
        else:
            print ("No input specified\nI'm lost")
        
        try:
            process(txtPath, args["RTmin"], args["RTmax"])
        except Exception as e:
            print ("Error: {}".format(repr(e)))
        
    else:
        print ("Error parsing command line")
